@@ -1,6 +1,6 @@
 #!@PERL_COMMAND@
 
-# $Id: smbldap-useradd.pl 138 2012-07-17 05:36:04Z fumiyas $
+# $Id: smbldap-useradd.pl 151 2012-10-24 02:04:28Z fumiyas $
 
 #  This code was developed by Jerome Tournier (jtournier@gmail.com) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -490,7 +490,11 @@ if (@userMailLocal) {
     push( @adds, 'mailLocalAddress' => [@userMailLocal] );
 }
 if (@userMailTo) {
-    push( @adds, 'mailRoutingAddress' => [@userMailTo] );
+    if (is_attr_single_value('mailRoutingAddress')) {
+        push(@adds, 'mailRoutingAddress' => join(',', @userMailTo));
+    } else {
+        push(@adds, 'mailRoutingAddress' => \@userMailTo);
+    }
 }
 
 # Custom modification - MPK
