@@ -1,6 +1,6 @@
 #!@PERL_COMMAND@
 
-# $Id: smbldap-usermod.pl 128 2012-03-05 08:00:09Z fumiyas $
+# $Id: smbldap-usermod.pl 155 2013-01-16 05:57:49Z fumiyas $
 
 #  This code was developped by Jerome Tournier (jtournier@gmail.com) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -471,8 +471,8 @@ if (defined($tmp = $Options{'M'})) {
 	  }
       }
     }
+    my @old_mail;
     if ($action) {
-        my @old_mail;
         @old_mail      = $user_entry->get_value('mail');
         if ( $action eq '+' ) {
             @mail          = &list_union( \@old_mail,      \@mail );
@@ -481,7 +481,11 @@ if (defined($tmp = $Options{'M'})) {
             @mail          = &list_minus( \@old_mail,      \@mail );
         }
     }
-    push( @mods, 'mail' => [@mail] );
+    if (@mail) {
+        push(@mods, 'mail' => \@mail);
+    } elsif (@old_mail) {
+        push(@dels, 'mail' => []);
+    }
 }
 
 my $mailobj = 0;
@@ -493,8 +497,8 @@ if (defined($tmp = $Options{'O'})) {
         $action = $1;
     }
     my @userMailLocal = &split_arg_comma($tmp);
+    my @old_MailLocal;
     if ($action) {
-        my @old_MailLocal;
         @old_MailLocal = $user_entry->get_value('mailLocalAddress');
         if ( $action eq '+' ) {
             @userMailLocal = &list_union( \@old_MailLocal, \@userMailLocal );
@@ -503,7 +507,11 @@ if (defined($tmp = $Options{'O'})) {
             @userMailLocal = &list_minus( \@old_MailLocal, \@userMailLocal );
         }
     }
-    push( @mods, 'mailLocalAddress', [@userMailLocal] );
+    if (@userMailLocal) {
+        push(@mods, 'mailLocalAddress' => \@userMailLocal);
+    } elsif (@old_MailLocal) {
+        push(@dels, 'mailLocalAddress' => []);
+    }
     $mailobj = 1;
 }
 
@@ -517,7 +525,9 @@ if (defined($tmp = $Options{'T'})) {
     }
     my @userMailTo = &split_arg_comma($tmp);
     if ($action) {
-        @old = $user_entry->get_value('mailRoutingAddress');
+        @old = map {
+            split_arg_comma($_);
+        } $user_entry->get_value('mailRoutingAddress');
     }
     if ( $action eq '+' ) {
         @userMailTo = &list_union( \@old, \@userMailTo );
@@ -525,7 +535,16 @@ if (defined($tmp = $Options{'T'})) {
     elsif ( $action eq '-' ) {
         @userMailTo = &list_minus( \@old, \@userMailTo );
     }
-    push( @mods, 'mailRoutingAddress', [@userMailTo] );
+
+    if (@userMailTo) {
+        if (is_attr_single_value('mailRoutingAddress')) {
+            push(@mods, 'mailRoutingAddress' => join(',', @userMailTo));
+        } else {
+            push(@mods, 'mailRoutingAddress' => \@userMailTo);
+        }
+    } elsif (@old) {
+        push(@dels, 'mailRoutingAddress' => []);
+    }
     $mailobj = 1;
 }
 if ($mailobj) {
@@ -785,9 +804,8 @@ my $modify = $ldap_master->modify( "$dn", 'replace' => {@mods} );
 $modify->code && warn "failed to modify entry: ", $modify->error;
 
 # we can delete only if @dels is not empty: we check the number of elements
-my $nb_to_del = scalar(@dels);
-if ( $nb_to_del != 0 ) {
-    $modify = $ldap_master->modify( "$dn", 'delete' => {@dels} );
+if (@dels) {
+    $modify = $ldap_master->modify($dn, 'delete' => {@dels});
 
     # don't proceed on error
     $modify->code && die "failed to modify entry: ", $modify->error;
