@@ -1,6 +1,6 @@
 #!@PERL_COMMAND@
 
-# $Id: smbldap_tools.pl 140 2012-08-07 11:11:49Z fumiyas $
+# $Id: smbldap_tools.pl 150 2012-10-24 02:04:10Z fumiyas $
 
 #  This code was developped by Jerome Tournier (jtournier@gmail.com) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -71,6 +71,7 @@ use vars qw(%config $ldap);
   is_unix_user
   is_nonldap_unix_user
   is_user_valid
+  is_attr_single_value
   does_sid_exist
   get_dn_from_line
   add_posix_machine
@@ -569,6 +570,18 @@ sub is_user_valid {
     return 1;
 }
 
+sub is_attr_single_value {
+    my $attr = shift;
+    my $schema_attr = $ldap->schema->attribute($attr);
+
+    unless ($schema_attr) {
+        warn "Attribute not found in LDAP schema: $attr";
+        return true;
+    }
+
+    return $schema_attr->{'single-value'};
+}
+
 # dn = get_dn_from_line ($dn_line)
 # helper to get "a=b,c=d" from "dn: a=b,c=d"
 sub get_dn_from_line {
@@ -808,7 +821,10 @@ sub read_user {
         foreach my $attr ( $entry->attributes ) {
             my @vals = $entry->get_value($attr);
 #	    my $val_utf8 = eval {
-#		Encode::decode_utf8($val, Encode::FB_CROAK);
+#               ## Use $tmp because Encode::FB_CROAK may change the input
+#               ## string. See Encode::FB_QUIET description in
+#               ## `perldoc Encode` for details.
+#		Encode::decode_utf8(my $tmp=$val, Encode::FB_CROAK);
 #	    };
 #	    $val = "**UNPRINTABLE**" if ($@ || $val_utf8 =~ /\P{IsPrint}/);
             $lines .= $attr . ": " . join( ',', @vals ) . "\n";
@@ -838,7 +854,10 @@ sub read_user_human_readable {
             my @vals = $entry->get_value($attr);
             foreach my $val (@vals) {
 		my $val_utf8 = eval {
-		    Encode::decode_utf8($val, Encode::FB_CROAK);
+                    ## Use $tmp because Encode::FB_CROAK may change the input
+                    ## string. See Encode::FB_QUIET description in
+                    ## `perldoc Encode` for details.
+		    Encode::decode_utf8(my $tmp=$val, Encode::FB_CROAK);
 		};
 		$val = "**UNPRINTABLE**" if ($@ || $val_utf8 =~ /\P{IsPrint}/);
             }
